open module org.jruby.jcodings {
    exports org.jcodings;
    exports org.jcodings.ascii;
    exports org.jcodings.constants;
    exports org.jcodings.exception;
    exports org.jcodings.specific;
    exports org.jcodings.spi;
    exports org.jcodings.transcode;
    exports org.jcodings.transcode.specific;
    exports org.jcodings.unicode;
    exports org.jcodings.util;
}